#include "stdafx.h"
#include "MyHorizontalSplitter.h"
#include "resource.h"


#define MY_HORIZONTAL_SPLITTER_OFFSET			2
#define MY_HORIZONTAL_SPLITTER_BORDER			8

LONG	CMyHorizontalSplitter::m_lCount	= 0;
HPEN	CMyHorizontalSplitter::m_hPen		= NULL;


//- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
CMyHorizontalSplitter::CMyHorizontalSplitter() : CMySplitterWindow(L"_My_Horizontal_Splitter_", IDC_HORIZONTAL_RESIZE_CUR)
{
}

//- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
CMyHorizontalSplitter::~CMyHorizontalSplitter()
{
}

//- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
BOOL CMyHorizontalSplitter::GetLeftRect(RECT* pRect)
{
	if ((pRect == NULL) || (m_hWnd == NULL))
		return(FALSE);

	GetClientRect(pRect);

	pRect->bottom = m_nCurrentPosition - MY_HORIZONTAL_SPLITTER_OFFSET;

	return(TRUE);
}

//- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
BOOL CMyHorizontalSplitter::GetRightRect(RECT* pRect)
{
	if ((pRect == NULL) || (m_hWnd == NULL))
		return(FALSE);

	GetClientRect(pRect);

	pRect->top = m_nCurrentPosition + MY_HORIZONTAL_SPLITTER_OFFSET;

	return(TRUE);
}

//- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
BOOL CMyHorizontalSplitter::OnCreate(LPCREATESTRUCT psCreateStruct)
{
	if (InterlockedIncrement(&m_lCount) == 1) {
		m_hPen	= CreatePen(PS_SOLID, 2 * MY_HORIZONTAL_SPLITTER_OFFSET, GetSysColor(COLOR_BTNFACE));		
	}

	return(CMySplitterWindow::OnCreate(psCreateStruct));
}

//- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
BOOL CMyHorizontalSplitter::OnDestroy()
{
	if (InterlockedDecrement(&m_lCount) == 0) {
		DeleteObject(m_hPen);
		m_hPen	= NULL;		
	}

	return(CMySplitterWindow::OnDestroy());
}

/*
//- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
BOOL CMyHorizontalSplitter::OnLButtonDblClk(DWORD dwKeyIndicator, int nPositionX, int nPositionY)
{
	RECT rtClient;

	GetClientRect(&rtClient);
	if (m_nCurrentPosition == (rtClient.top + MY_HORIZONTAL_SPLITTER_BORDER)) {
		if (m_nSavedPosition == (rtClient.top + MY_HORIZONTAL_SPLITTER_BORDER))
			m_nSavedPosition = max((long)160, rtClient.top + MY_HORIZONTAL_SPLITTER_BORDER);
		m_nRealPosition = m_nCurrentPosition = max((long)m_nSavedPosition, rtClient.top + MY_HORIZONTAL_SPLITTER_BORDER);
	}
	else {
		m_nSavedPosition	= m_nCurrentPosition;
		m_nRealPosition	= m_nCurrentPosition	= rtClient.top + MY_HORIZONTAL_SPLITTER_BORDER;
	}

	ResizeChildView();

	return(CMyWindow::OnLButtonDblClk(dwKeyIndicator, nPositionX, nPositionY));
}
*/

//- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
BOOL CMyHorizontalSplitter::OnMouseMove(DWORD dwKeyIndicator, int nPositionX, int nPositionY)
{
	if (m_bSize) ResizeView(nPositionY, FALSE);
	return(CMySplitterWindow::OnMouseMove(dwKeyIndicator, nPositionX, nPositionY));
}

//- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
BOOL CMyHorizontalSplitter::OnPaint(HDC hDC)
{
	RECT	rtClient, rtRect;
	//HPEN	hPen  = CreatePen(PS_SOLID, 2 * MY_HORIZONTAL_SPLITTER_OFFSET, GetSysColor(COLOR_BTNFACE));
	//HPEN	hBlack	= CreatePen(PS_SOLID, 1, RGB(0, 0, 0));
	HPEN	hOldPen	= NULL;

	if (hDC == NULL) return(FALSE);

	GetClientRect(&rtClient);
	GetClientRect(&rtRect);

	if (m_hPen != NULL) hOldPen = (HPEN)SelectObject(hDC, m_hPen);

	MoveToEx(hDC, rtClient.left, m_nRealPosition, NULL);
	LineTo(hDC, rtClient.right, m_nRealPosition);

	SetRect(&rtClient, rtClient.left + (rtClient.right - rtClient.left) / 2 - 15, m_nRealPosition - 1, rtClient.left + (rtClient.right - rtClient.left) / 2 + 15, m_nRealPosition + 1);
	DrawEdge(hDC, &rtClient, BDR_RAISEDINNER, BF_RECT);	

	if (hOldPen != NULL) SelectObject(hDC, hOldPen);
	//if (hPen != NULL) DeleteObject(hPen);
	//if (hBlack != NULL) DeleteObject(hBlack);

	return(TRUE);
}

//- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
void CMyHorizontalSplitter::ResizeChildView()
{
	int	nPos;
	RECT	rtParent;
	RECT	rtClient;

	GetClientRect(&rtParent);

	if (m_nSplitType & MY_SPLITTER_WINDOW_SPLIT_TYPE_TOP_LEFT) {
		nPos = min(m_nCurrentPosition, (int)((rtParent.bottom - rtParent.top) - MY_HORIZONTAL_SPLITTER_BORDER));
		nPos = max(nPos, (int)rtParent.top + MY_HORIZONTAL_SPLITTER_BORDER);
	}
	else if (m_nSplitType & MY_SPLITTER_WINDOW_SPLIT_TYPE_BOTTOM_RIGHT) {
		nPos = min((int)rtParent.bottom - m_nCurrentPosition, (int)((rtParent.bottom - rtParent.top) - MY_HORIZONTAL_SPLITTER_BORDER));
		nPos = max(nPos, (int)rtParent.top + MY_HORIZONTAL_SPLITTER_BORDER);
	}

	m_nRealPosition = nPos;

	if (m_pLeftView != NULL) {
		//SetWindowPos(m_pLeftView->GetHandle(), NULL, rtParent.left, rtParent.top, rtParent.right - rtParent.left, rtParent.bottom - rtParent.top - (nPos + 1), SWP_NOZORDER | SWP_FRAMECHANGED);
		//m_pLeftView->SetWindowPos(NULL, rtParent.left, rtParent.top,  rtParent.right - rtParent.left, nPos - 1, SWP_NOZORDER | SWP_FRAMECHANGED);
		SetRect(&rtClient, rtParent.left, rtParent.top, rtParent.right, nPos - MY_HORIZONTAL_SPLITTER_OFFSET);
		m_pLeftView->Resize(&rtClient);

	}
	if (m_pRightView != NULL) {
		//SetWindowPos(m_pRightView->GetHandle(), NULL, rtParent.left, rtParent.bottom - (nPos - 1), rtParent.right - rtParent.left, nPos - 1, SWP_NOZORDER | SWP_FRAMECHANGED);
		//m_pRightView->SetWindowPos(NULL, rtParent.left, rtParent.top + nPos + 1, rtParent.right - rtParent.left, rtParent.bottom - rtParent.top - (nPos + 1), SWP_NOZORDER | SWP_FRAMECHANGED);
		SetRect(&rtClient, rtParent.left, rtParent.top + nPos + MY_HORIZONTAL_SPLITTER_OFFSET, rtParent.right, rtParent.bottom);
		m_pRightView->Resize(&rtClient);
	}
}

//- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
void CMyHorizontalSplitter::ResizeView(int nPositionY, BOOL bEndDrag)
{
	RECT  rtClient;
	HDC   hDC = ::GetDC(m_hWnd);
	int   nOldMode;

	GetClientRect(&rtClient);

	if (bEndDrag != TRUE) {
		if (nPositionY < (rtClient.top + MY_HORIZONTAL_SPLITTER_BORDER)) nPositionY = rtClient.top + MY_HORIZONTAL_SPLITTER_BORDER;
		else if (nPositionY > (rtClient.bottom - MY_HORIZONTAL_SPLITTER_BORDER)) nPositionY = rtClient.bottom - MY_HORIZONTAL_SPLITTER_BORDER;
		//m_nCurrentPosition = (rtClient.bottom - rtClient.top) - nPositionY;
		if (m_nSplitType & MY_SPLITTER_WINDOW_SPLIT_TYPE_TOP_LEFT)
			m_nSavedPosition = m_nRealPosition = m_nCurrentPosition = nPositionY;
		else if (m_nSplitType & MY_SPLITTER_WINDOW_SPLIT_TYPE_BOTTOM_RIGHT)
			m_nSavedPosition = m_nRealPosition = m_nCurrentPosition = rtClient.bottom - nPositionY;

	}

	if (hDC == NULL) return;

	nOldMode = ::GetROP2(hDC);

	SetROP2(hDC, R2_NOTXORPEN);

	if (m_nLastPosition >= 0) {
		MoveToEx(hDC, rtClient.left, m_nLastPosition, NULL);
		LineTo(hDC, rtClient.right, m_nLastPosition);
	}
	if (bEndDrag != TRUE) {
		MoveToEx(hDC, rtClient.left, nPositionY, NULL);
		LineTo(hDC, rtClient.right, nPositionY);
	}

	m_nLastPosition = nPositionY;
	SetROP2(hDC, nOldMode);
	if (!::ReleaseDC(m_hWnd, hDC)) {
		assert(0);
	}
}