#include "stdafx.h"
#include "MyVerticalSplitter.h"
#include "resource.h"

#define MY_VERTICAL_SPLITTER_OFFSET			2
#define MY_VERTICAL_SPLITTER_BORDER			8

LONG	CMyVerticalSplitter::m_lCount	= 0;
HPEN	CMyVerticalSplitter::m_hPen	= NULL;


//- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
CMyVerticalSplitter::CMyVerticalSplitter() : CMySplitterWindow(L"_My_Vertical_Splitter_", IDC_VERTICAL_RESIZE_CUR)
{
}

//- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
CMyVerticalSplitter::~CMyVerticalSplitter()
{
}

//- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
BOOL CMyVerticalSplitter::GetLeftRect(RECT* pRect)
{
	if ((pRect == NULL) || (m_hWnd == NULL))
		return(FALSE);

	GetClientRect(pRect);

	pRect->right = m_nCurrentPosition - MY_VERTICAL_SPLITTER_OFFSET;

	return(TRUE);
}

//- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
BOOL CMyVerticalSplitter::GetRightRect(RECT* pRect)
{
	if ((pRect == NULL) || (m_hWnd == NULL))
		return(FALSE);

	GetClientRect(pRect);

	pRect->left = m_nCurrentPosition + MY_VERTICAL_SPLITTER_OFFSET;

	return(TRUE);
}

//- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
BOOL CMyVerticalSplitter::OnCreate(LPCREATESTRUCT psCreateStruct)
{
	if (InterlockedIncrement(&m_lCount) == 1) {
		m_hPen	= CreatePen(PS_SOLID, 2 * MY_VERTICAL_SPLITTER_OFFSET, GetSysColor(COLOR_BTNFACE));		
	}

	return(CMySplitterWindow::OnCreate(psCreateStruct));
}

//- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
BOOL CMyVerticalSplitter::OnDestroy()
{
	if (InterlockedDecrement(&m_lCount) == 0) {
		DeleteObject(m_hPen);
		m_hPen	= NULL;
	}

	return(CMySplitterWindow::OnDestroy());
}


/*
//- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
BOOL CMyVerticalSplitter::OnLButtonDblClk(DWORD dwKeyIndicator, int nPositionX, int nPositionY)
{
	RECT rtClient;

	GetClientRect(&rtClient);
	if (m_nCurrentPosition == (rtClient.left + MY_VERTICAL_SPLITTER_BORDER)) {
		if (m_nSavedPosition == (rtClient.left + MY_VERTICAL_SPLITTER_BORDER))
			m_nSavedPosition = min((long)160, rtClient.right - MY_VERTICAL_SPLITTER_BORDER);
		m_nRealPosition = m_nCurrentPosition = min((long)m_nSavedPosition, rtClient.right - MY_VERTICAL_SPLITTER_BORDER);;
	}
	else {
		m_nSavedPosition	= m_nCurrentPosition;
		m_nRealPosition	= m_nCurrentPosition	= rtClient.left + MY_VERTICAL_SPLITTER_BORDER;
	}

	ResizeChildView();

	return(CMyWindow::OnLButtonDblClk(dwKeyIndicator, nPositionX, nPositionY));
}
*/

//- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
BOOL CMyVerticalSplitter::OnMouseMove(DWORD dwKeyIndicator, int nPositionX, int nPositionY)
{
	if (m_bSize) ResizeView(nPositionX, FALSE);
	return(CMySplitterWindow::OnMouseMove(dwKeyIndicator, nPositionX, nPositionY));
}

//- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
BOOL CMyVerticalSplitter::OnPaint(HDC hDC)
{
	RECT        rtClient, rtRect;
	//HPEN        hPen  	= CreatePen(PS_SOLID, 2 * MY_VERTICAL_SPLITTER_OFFSET, GetSysColor(COLOR_BTNFACE));
	//HPEN			hBlack	= CreatePen(PS_SOLID, 1, RGB(0, 0, 0));
	HPEN			hOldPen	= NULL;

	if (hDC == NULL) return(FALSE);

	GetClientRect(&rtClient);
	GetClientRect(&rtRect);

	if (m_hPen != NULL) hOldPen = (HPEN)SelectObject(hDC, m_hPen);

	MoveToEx(hDC, m_nRealPosition, rtClient.top, NULL);
	LineTo(hDC, m_nRealPosition, rtClient.bottom);

	SetRect(&rtClient, m_nRealPosition - 1, rtClient.top + (rtClient.bottom - rtClient.top) / 2 - 15, m_nRealPosition + 1, rtClient.top + (rtClient.bottom - rtClient.top) / 2 + 15);
	//SetRect(&rtClient, m_nCurrentPosition - 1, rtClient.top + 2, m_nCurrentPosition + 1, rtClient.bottom - 2);
	DrawEdge(hDC, &rtClient, BDR_RAISEDINNER, BF_RECT);


	if (hOldPen != NULL) SelectObject(hDC, hOldPen);
	//if (hPen != NULL) DeleteObject(hPen);
	//if (hBlack != NULL) DeleteObject(hBlack);

	return(TRUE);
}

//- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
void CMyVerticalSplitter::ResizeChildView()
{
	int	nPos;
	RECT	rtParent;
	RECT	rtClient;

	GetClientRect(&rtParent);

	if (m_nSplitType & MY_SPLITTER_WINDOW_SPLIT_TYPE_TOP_LEFT) {
		nPos = min(m_nCurrentPosition, (int)((rtParent.right - rtParent.left) - MY_VERTICAL_SPLITTER_BORDER));
		nPos = max(nPos, (int)rtParent.left + MY_VERTICAL_SPLITTER_BORDER);
	}
	else if (m_nSplitType & MY_SPLITTER_WINDOW_SPLIT_TYPE_BOTTOM_RIGHT) {
		nPos = min((int)rtParent.right - m_nCurrentPosition, (int)((rtParent.right - rtParent.left) - MY_VERTICAL_SPLITTER_BORDER));
		nPos = max(nPos, (int)rtParent.left + MY_VERTICAL_SPLITTER_BORDER);
	}

	m_nRealPosition = nPos;

	if (m_pLeftView != NULL) {
		SetRect(&rtClient, 0, 0, nPos - MY_VERTICAL_SPLITTER_OFFSET, rtParent.bottom - rtParent.top);
		m_pLeftView->Resize(&rtClient);
		//m_pLeftView->SetWindowPos(NULL, 0, 0, nPos - 1, rtParent.bottom - rtParent.top, SWP_NOZORDER | SWP_FRAMECHANGED);
	}
	if (m_pRightView != NULL) {
		SetRect(&rtClient, nPos + MY_VERTICAL_SPLITTER_OFFSET, 0, rtParent.right - rtParent.left, rtParent.bottom - rtParent.top);
		m_pRightView->Resize(&rtClient);
		//m_pRightView->SetWindowPos(NULL, nPos + 1, 0, rtParent.right - rtParent.left - (nPos + 1), rtParent.bottom - rtParent.top, SWP_NOZORDER | SWP_FRAMECHANGED);
	}
}

//- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
void CMyVerticalSplitter::ResizeView(int nPositionX, BOOL bEndDrag)
{
	RECT  rtClient;
	HDC   hDC = ::GetDC(m_hWnd);
	int   nOldMode;

	GetClientRect(&rtClient);

	if (bEndDrag != TRUE) {
		if (nPositionX < (rtClient.left + MY_VERTICAL_SPLITTER_BORDER)) nPositionX = rtClient.left + MY_VERTICAL_SPLITTER_BORDER;
		else if (nPositionX > (rtClient.right - MY_VERTICAL_SPLITTER_BORDER)) nPositionX = rtClient.right - MY_VERTICAL_SPLITTER_BORDER;
		if (m_nSplitType & MY_SPLITTER_WINDOW_SPLIT_TYPE_TOP_LEFT)
			m_nSavedPosition = m_nRealPosition = m_nCurrentPosition = nPositionX;
		else if (m_nSplitType & MY_SPLITTER_WINDOW_SPLIT_TYPE_BOTTOM_RIGHT)
			m_nSavedPosition = m_nRealPosition = m_nCurrentPosition = rtClient.right - nPositionX;

	}

	if (hDC == NULL) return;

	nOldMode = ::GetROP2(hDC);

	SetROP2(hDC, R2_NOTXORPEN);

	if (m_nLastPosition >= 0) {
		MoveToEx(hDC, m_nLastPosition, rtClient.top, NULL);
		LineTo(hDC, m_nLastPosition, rtClient.bottom);
	}
	if (bEndDrag != TRUE) {
		MoveToEx(hDC, nPositionX, rtClient.top, NULL);
		LineTo(hDC, nPositionX, rtClient.bottom);
	}

	m_nLastPosition = nPositionX;

	SetROP2(hDC, nOldMode);

	if (!::ReleaseDC(m_hWnd, hDC)) {
		assert(0);
	}
}