#include "stdafx.h"
#include "ListViewPanel.h"
#include "resource.h"


//- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
CListViewPanel::CListViewPanel() : CMyPanel()
{		
	m_hImageList		= NULL;
	m_sItemList.nType	= 0;
	m_sItemList.pNext	= NULL;
	wcscpy_s(m_sItemList.wzName, MAX_PATH, L"");
}

//- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
CListViewPanel::~CListViewPanel()
{		
	if (m_hImageList != NULL) {
		ImageList_Destroy(m_hImageList);
		m_hImageList = NULL;
	}

	FreeList();
}

//- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
BOOL CListViewPanel::OnCreate(LPCREATESTRUCT psCreateStruct)
{
	RECT	rtRect;	

	GetClientRect(&rtRect);

	m_hListView.CreateMyEx(L"", WS_CHILD | WS_VISIBLE | LVS_REPORT, WS_EX_CLIENTEDGE, this, ID_LIST_VIEW, 0, 0, &rtRect);

	InitImageList();	
	InitListView();

	return(TRUE);
}

//- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
BOOL CListViewPanel::OnEraseBkgnd(HDC hDC)
{
	return(TRUE);
}

//- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
BOOL CListViewPanel::OnNotify(WPARAM wParam, LPARAM lParam)
{		
	LPNMHDR			pNMHDR		= (LPNMHDR)lParam;
	LPNMLISTVIEW	pListView	= (LPNMLISTVIEW)lParam;

	switch(pNMHDR->code) {
		case(LVN_ITEMACTIVATE):			
			OnItemActivate(pListView->iItem);			
			break;
	};	

	return(CMyWindow::OnNotify(wParam, lParam));
}

//- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
void CListViewPanel::InsertBitmap(UINT nID)
{
	HBITMAP	hBitmap;

	hBitmap	= LoadBitmap(GetSafeInstance(), MAKEINTRESOURCE(nID));

	if (hBitmap != NULL) {
		ImageList_AddMasked(m_hImageList, hBitmap, RGB(0, 0, 0));

		DeleteObject(hBitmap);
	}
}

//- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
void CListViewPanel::InitImageList()
{
	m_hImageList = ImageList_Create(16, 16, ILC_COLOR32 | ILC_MASK, 5, 0);

	InsertBitmap(IDB_FOLDER_C);
	InsertBitmap(IDB_FILE);
}

//- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
void CListViewPanel::InitListView()
{	
	// ukazka toho, jak se da vyuzit primy pristup k API funkcnim
	ListView_SetImageList(m_hListView.GetSafeHandle(), m_hImageList, LVSIL_SMALL);

	//m_hListView.SetExtendedStyle(LVS_EX_FULLROWSELECT | LVS_EX_GRIDLINES, LVS_EX_FULLROWSELECT | LVS_EX_GRIDLINES);

	m_hListView.InsertColumn(0, L"Nzev", 500);
}

//- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
void CListViewPanel::ChangeFolder(LPCWSTR pwzFolder)
{
	HANDLE				hFind;
	wchar_t				wzPath[MAX_PATH];
	WIN32_FIND_DATA	sFindData;
	//BOOL					bRet		= TRUE;
	int					nCount	= 0;
	LIST_VIEW_STRUCT*	pItem;

	m_hListView.DeleteAllItems();
	ClearList();

	if (pwzFolder == NULL) return;

	wcscpy_s(wzPath, MAX_PATH, pwzFolder);
	wcscat_s(wzPath, MAX_PATH, L"*.*");
	hFind = FindFirstFile(wzPath, &sFindData);

	if (hFind != INVALID_HANDLE_VALUE) {
		do {
			if ((sFindData.dwFileAttributes & FILE_ATTRIBUTE_DIRECTORY) == FILE_ATTRIBUTE_DIRECTORY) {
				if  ((wcscmp(sFindData.cFileName, L".") != 0)) {// && (wcscmp(sFindData.cFileName, L"..") != 0)) {
					pItem = InsertListItem(sFindData.cFileName, 0xFF);
					m_hListView.InsertItem(nCount++, sFindData.cFileName, 0, (LPVOID)pItem);
				}
			}
			else {
				pItem = InsertListItem(sFindData.cFileName, 0x01);
				m_hListView.InsertItem(nCount++, sFindData.cFileName, 1, (LPVOID)pItem);
			}
		} while(FindNextFile(hFind, &sFindData));		

		FindClose(hFind);
	}	

	m_hListView.SortItems(CompareFunc, NULL);
}

//- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
BOOL CListViewPanel::Resize(RECT* pClientRect)
{
	BOOL	bRet = CMyPanel::Resize(pClientRect);
	RECT	rtRect;

	GetClientRect(&rtRect);

	m_hListView.Resize(&rtRect);

	return(bRet);
}

//- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
void CListViewPanel::FreeList()
{
	LIST_VIEW_STRUCT*	pNext	= m_sItemList.pNext;
	LIST_VIEW_STRUCT*	pTemp;

	while(pNext != NULL) {
		pTemp = pNext->pNext;

		delete pNext;

		pNext = pTemp;
	}

	m_sItemList.nType	= 0;
	m_sItemList.pNext	= NULL;
	wcscpy_s(m_sItemList.wzName, MAX_PATH, L"");
}

//- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
LIST_VIEW_STRUCT* CListViewPanel::InsertListItem(LPCWSTR pwzName, BYTE nType)
{
	LIST_VIEW_STRUCT*	pNext	= &m_sItemList;
	LIST_VIEW_STRUCT*	pLast;

	assert(pwzName != NULL);

	while(pNext != NULL) {
		pLast = pNext;

		if (pNext->nType == 0) {
			wcscpy_s(pNext->wzName, MAX_PATH, pwzName);
			pNext->nType = nType;
			return(pNext);
		}

		pNext = pNext->pNext;
	}

	assert(pNext == NULL);
	assert(pLast != NULL);
	assert(pLast->pNext == NULL);

	if ((pLast->pNext = new LIST_VIEW_STRUCT) == NULL) {
		assert(0);
		return(NULL);
	}

	pNext				= pLast->pNext;
	pNext->pNext	= NULL;
	pNext->nType	= nType;
	wcscpy_s(pNext->wzName, MAX_PATH, pwzName);

	return(pNext);
}

//- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
void CListViewPanel::ClearList()
{
	LIST_VIEW_STRUCT*	pNext	= &m_sItemList;
	
	while(pNext != NULL) {
		wcscpy_s(pNext->wzName, MAX_PATH, L"");
		pNext->nType = 0;
		pNext = pNext->pNext;
	}
}

//- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
int CALLBACK CListViewPanel::CompareFunc(LPARAM lParam1, LPARAM lParam2, LPARAM lParamSort)
{
	LIST_VIEW_STRUCT*	pItem1	= (LIST_VIEW_STRUCT*)lParam1;
	LIST_VIEW_STRUCT*	pItem2	= (LIST_VIEW_STRUCT*)lParam2;

	assert(pItem1 != NULL);
	assert(pItem2 != NULL);

	// adresare jsou vzdy nad soubory
	if (pItem1->nType < pItem2->nType) return(1);
	if (pItem1->nType > pItem2->nType) return(-1);	

	// specialni adresar ".." je vzdy prvni
	if (wcscmp(pItem1->wzName, L"..") == 0) return(-1);
	if (wcscmp(pItem2->wzName, L"..") == 0) return(1);

	// jinak tridime podle jmena souboru/adresaru
	return(_wcsicmp(pItem1->wzName, pItem2->wzName));
}

//- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
void CListViewPanel::OnItemActivate(int nItemIndex)
{
	LVITEM				sItem;
	LIST_VIEW_STRUCT*	pItem;

	sItem.mask		= LVIF_PARAM;
	sItem.iItem		= nItemIndex;
	sItem.iSubItem	= 0;

	m_hListView.GetItem(&sItem);

	pItem = (LIST_VIEW_STRUCT*)sItem.lParam;

	// jedna se o adresar
	if (pItem->nType == 0xFF) {
		::SendMessage(GetParent(m_hWnd), WM_COMMAND, MAKEWPARAM(ID_LIST_VIEW, 1), (LPARAM)pItem->wzName);
	}
}