#include "stdafx.h"
#include "TreeViewPanel.h"
#include "resource.h"


//- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
CTreeViewPanel::CTreeViewPanel() : CMyPanel()
{	
	m_hRoot			= NULL;
	m_hImageList	= NULL;
}

//- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
CTreeViewPanel::~CTreeViewPanel()
{		
	if (m_hImageList != NULL) {
		ImageList_Destroy(m_hImageList);
		m_hImageList = NULL;
	}
}

//- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
BOOL CTreeViewPanel::OnCreate(LPCREATESTRUCT psCreateStruct)
{
	RECT	rtRect;	

	GetClientRect(&rtRect);

	m_hTreeView.CreateMyEx(L"", WS_CHILD | WS_VISIBLE | TVS_LINESATROOT | TVS_HASLINES | TVS_HASBUTTONS, WS_EX_CLIENTEDGE, this, ID_TREE_VIEW, 0, 0, &rtRect);

	InitImageList();	
	InitTreeView();

	return(TRUE);
}

//- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
BOOL CTreeViewPanel::OnEraseBkgnd(HDC hDC)
{
	return(TRUE);
}

//- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
BOOL CTreeViewPanel::OnNotify(WPARAM wParam, LPARAM lParam)
{
	LPNMHDR			pNMHDR		= (LPNMHDR)lParam;
	LPNMTREEVIEW	pTreeView	= (LPNMTREEVIEW)lParam;

	switch(pNMHDR->code) {
		case(TVN_SELCHANGED):			
			OnSelChanged(pTreeView->itemNew.hItem);
			break;
		case(TVN_ITEMEXPANDING):
			if (pTreeView->action == TVE_EXPAND) OnExpanding(pTreeView->itemNew.hItem);
			break;
	};

	return(CMyWindow::OnNotify(wParam, lParam));
}

//- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
void CTreeViewPanel::InsertBitmap(UINT nID)
{
	HBITMAP	hBitmap;

	hBitmap	= LoadBitmap(GetSafeInstance(), MAKEINTRESOURCE(nID));

	if (hBitmap != NULL) {
		ImageList_AddMasked(m_hImageList, hBitmap, RGB(0, 0, 0));

		DeleteObject(hBitmap);
	}
}

//- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
void CTreeViewPanel::InitImageList()
{
	m_hImageList = ImageList_Create(16, 16, ILC_COLOR32 | ILC_MASK, 5, 0);

	InsertBitmap(IDB_PC);
	InsertBitmap(IDB_FLOPPY);
	InsertBitmap(IDB_DISK);
	InsertBitmap(IDB_CD);
	InsertBitmap(IDB_FOLDER_C);
	InsertBitmap(IDB_FOLDER_O);
}

//- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
void CTreeViewPanel::InitTreeView()
{	
	wchar_t			wzName[MAX_COMPUTERNAME_LENGTH + 1];
	TVINSERTSTRUCT	sInsert;
	HTREEITEM		hDisk;
	DWORD				dwLength	= MAX_COMPUTERNAME_LENGTH;
	UINT				nType;

	m_hTreeView.SetImageList(m_hImageList, TVSIL_NORMAL);

	GetComputerName(wzName, &dwLength);

	sInsert.hInsertAfter				= TVI_ROOT;
	sInsert.hParent					= NULL;
	sInsert.item.mask					= TVIF_TEXT | TVIF_IMAGE | TVIF_SELECTEDIMAGE;
	sInsert.item.pszText				= wzName;
	sInsert.item.iImage				= 0;
	sInsert.item.iSelectedImage	= 0;

	m_hRoot = m_hTreeView.InsertItem(&sInsert);


	for (wchar_t cDisk = 'A';cDisk <= 'Z';cDisk++) {
		wzName[0]	= cDisk;
		wzName[1]	= ':';
		//wzName[2]	= '\\';
		wzName[2]	= '\0';

		nType			= GetDriveType(wzName);

		if ((nType != DRIVE_UNKNOWN) && (nType != DRIVE_NO_ROOT_DIR)) {
			switch(nType) {
				case(DRIVE_REMOVABLE):
					sInsert.item.iImage				= 1;
					sInsert.item.iSelectedImage	= 1;
					break;
				case(DRIVE_FIXED):
					sInsert.item.iImage				= 2;
					sInsert.item.iSelectedImage	= 2;
					break;
				case(DRIVE_REMOTE):
					sInsert.item.iImage				= 2;
					sInsert.item.iSelectedImage	= 2;
					break;
				case(DRIVE_CDROM):
					sInsert.item.iImage				= 3;
					sInsert.item.iSelectedImage	= 3;
					break;
				case(DRIVE_RAMDISK):
					sInsert.item.iImage				= 2;
					sInsert.item.iSelectedImage	= 2;
					break;
			};

			sInsert.hInsertAfter	= TVI_LAST;
			sInsert.hParent		= m_hRoot;
			sInsert.item.mask		= TVIF_TEXT | TVIF_IMAGE | TVIF_SELECTEDIMAGE;
			sInsert.item.pszText	= wzName;

			hDisk = m_hTreeView.InsertItem(&sInsert);	

			sInsert.hInsertAfter	= TVI_LAST;
			sInsert.hParent		= hDisk;
			sInsert.item.mask		= TVIF_TEXT | TVIF_IMAGE | TVIF_SELECTEDIMAGE;
			sInsert.item.pszText	= L"_dummy_";
			m_hTreeView.InsertItem(&sInsert);
		}
	}

	m_hTreeView.Expand(m_hRoot, TVE_EXPAND);
}

//- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
void CTreeViewPanel::OnSelChanged(HTREEITEM hItem)
{	
	wchar_t	wzItem[MAX_PATH];
	TVITEM	sItem;

	if (hItem == NULL) return;

	if (hItem == m_hRoot) {
		::SendMessage(GetParent(m_hWnd), WM_COMMAND, MAKEWPARAM(ID_TREE_VIEW, 1), (LPARAM)NULL);
		return;
	}

	wcscpy_s(m_wzPath, MAX_PATH, L"");

	while(hItem != m_hRoot) {
		sItem.hItem			= hItem;
		sItem.mask			= TVIF_HANDLE | TVIF_TEXT;
		sItem.pszText		= wzItem;
		sItem.cchTextMax	= MAX_PATH;

		m_hTreeView.GetItem(&sItem);

		wcscat_s(wzItem, MAX_PATH, L"\\");
		wcscat_s(wzItem, MAX_PATH, m_wzPath);
		wcscpy_s(m_wzPath, MAX_PATH, wzItem);

		hItem = m_hTreeView.GetParent(hItem);
	}

	::SendMessage(GetParent(m_hWnd), WM_COMMAND, MAKEWPARAM(ID_TREE_VIEW, 1), (LPARAM)m_wzPath);
}

//- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
void CTreeViewPanel::OnExpanding(HTREEITEM hItem)
{
	wchar_t				wzItem[MAX_PATH];
	wchar_t				wzPath[MAX_PATH];
	TVITEM				sItem;
	HANDLE				hFind;
	BOOL					bRet = TRUE;
	HTREEITEM			hCurrent	= hItem;
	WIN32_FIND_DATA	sFindData;
	TVINSERTSTRUCT		sInsert;

	if (hItem == NULL) return;

	if (hItem == m_hRoot) return;

	wcscpy_s(wzPath, MAX_PATH, L"*.*");

	while(hItem != m_hRoot) {
		sItem.hItem			= hItem;
		sItem.mask			= TVIF_HANDLE | TVIF_TEXT;
		sItem.pszText		= wzItem;
		sItem.cchTextMax	= MAX_PATH;

		m_hTreeView.GetItem(&sItem);

		wcscat_s(wzItem, MAX_PATH, L"\\");
		wcscat_s(wzItem, MAX_PATH, wzPath);
		wcscpy_s(wzPath, MAX_PATH, wzItem);

		hItem = m_hTreeView.GetParent(hItem);
	}

	while((hItem = m_hTreeView.GetChild(hCurrent)) != NULL)
		m_hTreeView.DeleteItem(hItem);	
	
	hFind = FindFirstFile(wzPath, &sFindData);
	if (hFind != INVALID_HANDLE_VALUE) {
		while(bRet) {
			if (((sFindData.dwFileAttributes & FILE_ATTRIBUTE_DIRECTORY) == FILE_ATTRIBUTE_DIRECTORY) && (wcscmp(sFindData.cFileName, L".") != 0) && (wcscmp(sFindData.cFileName, L"..") != 0)) {
				sInsert.hInsertAfter				= TVI_SORT;
				sInsert.hParent					= hCurrent;
				sInsert.item.mask					= TVIF_TEXT | TVIF_IMAGE | TVIF_SELECTEDIMAGE;
				sInsert.item.iImage				= 4;
				sInsert.item.iSelectedImage	= 5;
				sInsert.item.pszText				= sFindData.cFileName;

				hItem									= m_hTreeView.InsertItem(&sInsert);

				sInsert.hParent					= hItem;
				sInsert.item.pszText				= L"_dummy_";

				m_hTreeView.InsertItem(&sInsert);
			}

			bRet = FindNextFile(hFind, &sFindData);
		}

		FindClose(hFind);
	}
}

//- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
BOOL CTreeViewPanel::Resize(RECT* pClientRect)
{
	BOOL	bRet = CMyPanel::Resize(pClientRect);
	RECT	rtRect;

	GetClientRect(&rtRect);

	m_hTreeView.Resize(&rtRect);

	return(bRet);
}

//- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
void CTreeViewPanel::ChangeItem(LPCWSTR pwzItem)
{
	HTREEITEM	hItem;
	HTREEITEM	hNext;
	TVITEM		sItem;
	wchar_t		wzItem[MAX_PATH];

	assert(pwzItem != NULL);

	hItem = m_hTreeView.GetSelection();

	if (wcscmp(pwzItem, L"..") == 0) {
		hNext = m_hTreeView.GetParent(hItem);
		m_hTreeView.SelectItem(hNext);
		return;
	}

	m_hTreeView.Expand(hItem, TVE_EXPAND);
	//OnExpanding(hItem);

	hNext = m_hTreeView.GetChild(hItem);
	while(hNext != NULL) {

		sItem.mask			= TVIF_TEXT | TVIF_HANDLE;
		sItem.hItem			= hNext;
		sItem.pszText		= wzItem;
		sItem.cchTextMax	= MAX_PATH;

		if (m_hTreeView.GetItem(&sItem)) {
			if (wcscmp(wzItem, pwzItem) == 0) {
				m_hTreeView.SelectItem(hNext);
				//OnSelChanged(hNext);
				return;
			}
		}
		
		hNext = m_hTreeView.GetNextSibling(hNext);
	}
}